package indi.felix.easy.core.elastic;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import indi.felix.easy.core.elastic.client.ClientConfigProperties;
import indi.felix.easy.core.elastic.client.EasyEsService;
import indi.felix.easy.core.elastic.utool.Const;
import indi.felix.easy.core.elastic.utool.EsUtils;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.ElasticsearchException;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import indi.felix.easy.core.elastic.client.ClientManage;
import indi.felix.easy.core.elastic.rest.EasyRest;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.rest.RestStatus;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import static indi.felix.easy.core.elastic.utool.Const.DOC_TYPE;

public class EasyEs {
    /**
     * 默认集群名称
     */
    private final static String DEFAULT_CLUSTER = "elasticsearch";
    private final String HOST;
    private volatile RestHighLevelClient client;
    private volatile EasyEsService esService;
    private volatile EasyRest esRest;

    /**
     * 一个集群名称 创建一个实例
     */
//    private static ConcurrentHashMap<String, EasyEs> clientMap = new ConcurrentHashMap<>();

//    public static EasyEs getEasyESClient(String host){
//        if (StringUtils.isBlank(host)) {
//            throw new ElasticsearchException("Host not is Null");
//        }
//        EasyEs easyEs = clientMap.get(host);
//        if (easyEs == null) {
//            synchronized (EasyEs.class) {
//                if (easyEs == null) {
//                    easyEs = new EasyEs(host);
//                }
//            }
//        }
//
//        return easyEs;
//    }

//    public static EasyEs getEasyESClient(ClientConfigProperties params){
//        String host = params.getHost();
//        if (StringUtils.isBlank(host)) {
//            throw new ElasticsearchException("Host not is Null");
//        }
//        EasyEs easyEs = clientMap.get(host);
//        if (easyEs == null) {
//            synchronized (EasyEs.class) {
//                if (easyEs == null) {
//                    easyEs = new EasyEs(params);
//                }
//            }
//        }
//
//        return easyEs;
//    }

    RestHighLevelClient getSingleClient(String host) {
        if (StringUtils.isBlank(host)) {
            throw new ElasticsearchException("无效的es连接");
        }
        if (client == null) {
            synchronized (RestHighLevelClient.class) {
                if (client == null) {
                    client = ClientManage.getClient(host);
                }
            }
        }

        return client;
    }

    RestHighLevelClient getSingleClient(ClientConfigProperties params) {
        String host = params.getHost();
        if (StringUtils.isBlank(host)) {
            throw new ElasticsearchException("无效的es连接");
        }
        if (client == null) {
            synchronized (RestHighLevelClient.class) {
                if (client == null) {
                    client = ClientManage.getClient(params);
                }
            }
        }

        return client;
    }

    EasyEsService getEsService(RestHighLevelClient client) {
        if (esService == null) {
            synchronized (EasyEsService.class) {
                if (esService == null) {
                    esService = new EasyEsService(client);
                }
            }
        }

        return esService;
    }

    EasyRest getEsRest(RestHighLevelClient client) {
        if (esRest == null) {
            synchronized (EasyRest.class) {
                if (esRest == null) {
                    esRest = new EasyRest(client);
                }
            }
        }

        return esRest;
    }


    public String getHOST(){
        return HOST;
    }

    public EasyEs(String... host) {
        String newHost = String.join(",",host);
        HOST = newHost;
        client = getSingleClient(newHost);
    }



    public EasyEs(ClientConfigProperties params) {
        HOST = params.getHost();
        client = getSingleClient(params);
    }

    /**
     * 测试连接
     * @return
     * @throws IOException
     */
    public boolean testConn() throws IOException {
         return client.ping(RequestOptions.DEFAULT);
    }


    /**
     * 调用高级客户端search接口
     *
     * @return
     */
    public EasyEsService search() {

        return getEsService(client);
    }

    /**
     * 调用rest api接口
     *
     * @return
     */
    public EasyRest rest() {
        return getEsRest(client);
    }

//    public String getEsVersion(){
//        client.getLowLevelClient().
//        return responseBody(defaultMethod, newEndpoint);
//    }

    public void bulkAddDoc(String index, List<Map> objects) {
        BulkRequest request = new BulkRequest();

        if(!rest().indexExists(index)){
            throw new ElasticsearchException("index is not exists");
        }
        if (objects == null || objects.size() < 1) {
            throw new ElasticsearchException("mappers can not be empty");
        }

        search().bulkInsert(index, objects, 2000);
//
//        BulkResponse bulkResponse = null;
//        try {
//            if (objects == null || objects.size() < 1) {
//                throw new ElasticsearchException("mappers can not be empty");
//            }
//
//            for (int i = 0; i < objects.size(); i++) {
//                Map mapper = objects.get(i);
//                IndexRequest indexRequest = new IndexRequest(index);
//                EsUtils.setDefaultProperties(indexRequest, mapper);
//
//                request.add(indexRequest
//                        .opType("create")
//                        .source(JSON.toJSONString(mapper), XContentType.JSON));
//            }
//            bulkResponse = client.bulk(request, RequestOptions.DEFAULT);
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
//        RestStatus stat = bulkResponse.status();
    }

}
