package indi.felix.easy.core.elastic.client;

import indi.felix.easy.core.elastic.function.SearchHitsFunction;
import indi.felix.easy.core.elastic.model.MappingModel;
import indi.felix.easy.core.elastic.model.SettingModel;
import indi.felix.easy.core.elastic.utool.Const;
import indi.felix.easy.core.elastic.utool.DelEsListener;
import javafx.util.Pair;
import org.elasticsearch.action.search.*;
import org.elasticsearch.client.*;
import org.elasticsearch.common.unit.TimeValue;
import indi.felix.easy.core.exceptions.ElasticsearchException;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.*;
import org.elasticsearch.search.Scroll;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortOrder;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;

import static indi.felix.easy.core.elastic.utool.Const.DOC_TYPE;

/**
 * TODO 6.8版本以下不支持include_type_name
 * host [http://localhost:9200], URI [/cat?master_timeout=30sinclude_type_name=falsetimeout=30s], status line [HTTP/1.1 400 Bad Request]
 * {error:{root_cause:[{type:illegal_argument_exception,reason:request [/cat] contains unrecognized parameter: [include_type_name]}],type:illegal_argument_exception,reason:request [/cat] contains unrecognized parameter: [include_type_name]},status:400}
 */
public class EasyEsService extends AbstractService {

    public EasyEsService(RestHighLevelClient client) {
        super(client);
    }
//     todo 有待完善
//    public EasyEsServiceLambda lambda(String index) {
//        EasyEsServiceLambda lambda = new EasyEsServiceLambda(this.restHighLevelClient, index);
//
//        return lambda;
//    }

//    public EasyEsServiceDSL dsl() {
//        return new EasyEsServiceDSL();
//    }

    /**
     * 转换格式
     *
     * @param searchHits
     * @return
     */
//    todo 有待完善
//    public List<Map<String, Object>> toList(SearchHits searchHits) {
//
//        List<Map<String, Object>> sourceAsMap = new ArrayList<>();
//        for (SearchHit searchHit : searchHits) {
//            Map<String, Object> sourceAsMap1 = searchHit.getSourceAsMap();
//            sourceAsMap1.put(Const.ID, searchHit.getId());
//            sourceAsMap1.put(Const.ROUTING, searchHit.docId());
//            sourceAsMap1.put("index_name", searchHit.getIndex());
//            sourceAsMap.add(sourceAsMap1);
//        }
//        return sourceAsMap;
//    }

    /**
     * 判断索引是否存在
     *
     * @return
     * @throws Exception
     */

    public boolean indexExist(String... index) {
        return this.exist(index);
    }

    public boolean createIndex(String... index) {
        try {
            return createIndexHandle(index);
        } catch (Exception e) {
            throw new ElasticsearchException();
        }
    }

    public boolean createIndex(String index, String alias, SettingModel setting,
                               MappingModel properties) {
        try {
            return createIndexHandle(index, setting, properties);
        } catch (Exception e) {
            throw new ElasticsearchException();
        }
    }

    public boolean deleteIndex(String index) {
        return deleteIndexHandle(index);
    }

    public void deleteByQuery(String index, QueryBuilder queryBuilder) {
        deleteByQueryHandle(index, queryBuilder);
    }


    /**
     * 批量导入
     *
     * @param index     //     * @param isAutoId  使用自动id 还是使用传入对象的id
     * @param list
     * @param batchSize
     * @return ArrayList
     */
    public ArrayList<Pair<String, Object>> bulkInsert(String index, List<Map> list, int batchSize) {
        return insertBatch(index, list, batchSize);
    }


    public SearchResponse searchResponse(String index, SearchSourceBuilder builder) throws IOException {
        SearchRequest request = new SearchRequest(index);
        request.source(builder);
        SearchResponse response = restHighLevelClient.search(request, RequestOptions.DEFAULT);

        return response;
    }

    /**
     * 查询
     *
     * @param index   索引名称
     * @param builder 查询条件builder
     * @return
     * @throws IOException
     */
    public SearchHits search(String index, SearchSourceBuilder builder) throws IOException {
        SearchResponse response = searchResponse(index, builder);
        return response.getHits();
    }

    public SearchResponse searchResponse(String field, String key, int page, int size, String... indexNames) throws IOException {
        SearchRequest request = new SearchRequest(indexNames);
        SearchSourceBuilder builder = new SearchSourceBuilder();
        builder.query(new MatchQueryBuilder(field, key))
                .from(page)
                .size(size);
        request.source(builder);
        return restHighLevelClient.search(request, RequestOptions.DEFAULT);
    }

    /**
     * 简单模糊匹配 默认分页为 0,10
     *
     * @param field
     * @param key
     * @param page
     * @param size
     * @param indexNames
     * @return
     * @throws IOException
     */
    public SearchHits search(String field, String key, int page, int size, String... indexNames) throws IOException {
        SearchResponse response = searchResponse(field, key, page, size, indexNames);

        return response.getHits();
    }


    public SearchResponse searchResponsePage(String index, Integer form, Integer size) throws IOException {
        SearchRequest searchRequest = new SearchRequest(index);
        //构建搜索条件
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //QueryBuilders.matchAllQuery查询所有
        MatchAllQueryBuilder matchAllQueryBuilder = QueryBuilders.matchAllQuery();
        searchSourceBuilder.query(matchAllQueryBuilder);
        //构建分页
        if (form == 1) {
            searchSourceBuilder.from();
        } else {
            searchSourceBuilder.from(form);
        }
        searchSourceBuilder.size(size);
        searchSourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
        searchRequest.source(searchSourceBuilder);
        SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);

        return search;
    }

    /**
     * 根据分页查询数据 查询全部
     *
     * @param index 索引名称
     * @param form  页数 如果查看第一页传1
     * @param size  数量 默认传10
     * @return 查询好的SearchHit数组
     * @throws IOException es连接异常
     */
    public SearchHits searchPage(String index, Integer form, Integer size) throws IOException {
        SearchResponse search = searchResponsePage(index, form, size);
        /*List<Map<String, Object>> maps = toList(search.getHits());
        PageInfo<Map<String, Object>> mapPageInfo = new PageInfo<>(maps);*/
        //return mapPageInfo;
        return search.getHits();
    }

    /**
     * scroll 查询， 用于批量导出
     * 对于用户高并发访问的场景，不推荐用这种方式，scroll 更适用于批处理类的后台任务。
     * eg: scroll(String index, QueryBuilder query, (SearchHitsFunction<SearchHits, Map>) func{
     * for (SearchHit searchHit : searchHits) {
     * <p>
     * }
     * <p>
     * return map
     * })
     */
    public void scrollForResponse(String index, QueryBuilder query, Consumer<SearchResponse> consumer) throws IOException {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //返回字段信息
//        searchSourceBuilder.fetchSource(new String[]{"id", "x_dip","i_dport"}, new String[]{});
        searchSourceBuilder.query(query);
        scrollForResponse(index, searchSourceBuilder, consumer);
    }

    /**
     * 滚动查询
     * 对于用户高并发访问的场景，不推荐用这种方式，scroll 更适用于批处理类的后台任务。
     *
     * @param index
     * @param searchSourceBuilder
     * @param consumer
     * @throws IOException
     */
    public void scrollForResponse(String index, SearchSourceBuilder searchSourceBuilder, Consumer<SearchResponse> consumer) throws IOException {
        //设定滚动时间间隔
        final Scroll scroll = new Scroll(TimeValue.timeValueMinutes(60L));
        SearchRequest searchRequest = new SearchRequest(index);//索引库
        searchRequest.scroll(scroll);
        if(searchSourceBuilder.size() < 1000){
            //设定每次返回多少条数据
            searchSourceBuilder.size(1000);
        }

        searchRequest.source(searchSourceBuilder);

        SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
        consumer.accept(searchResponse);
        String scrollId = searchResponse.getScrollId();
        SearchHit[] searchHits = searchResponse.getHits().getHits();

        //遍历搜索命中的数据，直到没有数据
        while (searchHits != null && searchHits.length > 0) {

            SearchScrollRequest scrollRequest = new SearchScrollRequest(scrollId);
            scrollRequest.scroll(scroll);
            searchResponse = restHighLevelClient.scroll(scrollRequest, RequestOptions.DEFAULT);

            consumer.accept(searchResponse);

            scrollId = searchResponse.getScrollId();
            searchHits = searchResponse.getHits().getHits();
        }

        //清除滚屏
        ClearScrollRequest clearScrollRequest = new ClearScrollRequest();
        //也可以选择setScrollIds()将多个scrollId一起使用
        clearScrollRequest.addScrollId(scrollId);
        ClearScrollResponse clearScrollResponse = null;
        try {
            clearScrollResponse = restHighLevelClient.clearScroll(clearScrollRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            e.printStackTrace();
        }
        boolean succeeded = clearScrollResponse.isSucceeded();
    }


    public void scrollForHit(String index, QueryBuilder query, Consumer<SearchHit> consumer) throws IOException {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(query);
        scrollForHit(index, searchSourceBuilder, consumer);
    }

    public void scrollForHit(String index, SearchSourceBuilder searchSourceBuilder, Consumer<SearchHit> consumer) throws IOException {
        scrollForResponse(index, searchSourceBuilder, searchResponse -> forEachHits(searchResponse, consumer));
    }

    /**
     * searchAfter 分页查询
     * search after因为每一页的数据依赖于上一页最后一条数据，所以无法跳页请求
     * 需要排序- 需要排序，官方（5.6版本）推荐使用 _uid 作为排序字段和全局唯一值（
     * 注：不同版本的ES，官方文档有变动，请参考对应版本的文档。
     * 比如，7.x版本建议拷贝_id字段值到另外一个支持doc value的字段，用于作为排序字段）；
     * 不适合跳页需求
     * 在7.x官方文档中，已不太建议使用Scroll API来深分页了
     * 通过实时cursor来规避耗时和占存储的性能问题，基于上一页的结果帮助检索下一页；
     * 使用search_after参数时，from的值必须为0或者-1
     * 性能： search_after > scroll > from + size
     * search_after： 海量数据分页
     *
     * @param index
     * @param builder
     * @param consumer
     * @throws IOException
     */
    public void searchForResponse(String index, SearchSourceBuilder builder, Consumer<SearchResponse> consumer) throws IOException {
        SearchRequest searchRequest = new SearchRequest(index);

        searchRequest.source(builder);

        SearchSourceBuilder sourceBuilder = searchRequest.source();
        sourceBuilder.from(0);
        sourceBuilder.size(1000); //在构建查询条件时，即可设置大小
        //设置排序字段， 先判断有没有id字段， 如果没有就用_uid
//        sourceBuilder.sort(Const.ID, SortOrder.ASC);
        sourceBuilder.sort(Const.UID, SortOrder.ASC);
        SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
        SearchHit[] searchHits = searchResponse.getHits().getHits();
        while (searchHits.length > 0) {
            consumer.accept(searchResponse);
            SearchHit last = searchHits[searchHits.length - 1];
            sourceBuilder.searchAfter(last.getSortValues());
            searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            searchHits = searchResponse.getHits().getHits();
        }
    }

    public void searchForResponse(String index, QueryBuilder query, Consumer<SearchResponse> consumer) throws IOException {
        SearchSourceBuilder searchBuilder = new SearchSourceBuilder();
        searchBuilder.query(query);

        searchForResponse(index, searchBuilder, consumer);
    }


    public void searchForHit(String index, SearchSourceBuilder builder,
                             Consumer<SearchHit> consumer) throws IOException {
        searchForResponse(index, builder, searchResponse -> forEachHits(searchResponse, consumer));
    }

    public void searchForHit(String index, QueryBuilder query,
                             Consumer<SearchHit> consumer) throws IOException {
        searchForResponse(index, query, searchResponse -> forEachHits(searchResponse, consumer));
    }

//    public static void searchForHit(SearchRequestBuilder requestBuilder, Consumer<SearchHit> consumer) {
//        searchForResponse(requestBuilder, searchResponse -> {
//            SearchHit[] searchHits = searchResponse.getHits().getHits();
//            for (SearchHit searchHit : searchHits) {
//                consumer.accept(searchHit);
//            }
//        });
//    }

    public void forEachHits(SearchResponse searchResponse, Consumer<SearchHit> consumer) {
        if (searchResponse == null) {
            return;
        }
        SearchHit[] searchHits = searchResponse.getHits().getHits();
        for (SearchHit searchHit : searchHits) {
            consumer.accept(searchHit);
        }
    }

    /**
     * 分词查找
     *
     * @param index 索引名称
     * @param form  页数 如果查看第一页传1
     * @param size  数量 默认传10
     * @param name  查询参数名
     * @param value 查询值
     * @return 查询好的SearchHit数组
     * @throws IOException es连接异常
     */
    public SearchHit[] boolPage(String index, Integer form, Integer size, String name, String value) throws IOException {
        SearchRequest searchRequest = new SearchRequest(index);
        //构建搜索条件
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //QueryBuilders.termQuery(); //精确查找
        /*TermQueryBuilder termQueryBuilder = QueryBuilders.termQuery(name, value);
        searchSourceBuilder.query(termQueryBuilder);*/

        //使用bool检索
        /*bool:几种格式
        must:所有分句都必须匹配，与 AND 相同。
        must_not:所有分句都必须不匹配，与 NOT 相同。
        should: 至少有一个分句匹配，与 OR 相同。
        1 term查询（精准查询）
        2 math查询（分词匹配查询）
        3 fuzzy查询（模糊查询）
        4 wildcard(通配符查询)
        5 bool查询（布尔查询）
        */
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery()
                .should(QueryBuilders.matchQuery(name, value)
                        .analyzer("ik_max_word")
                        .operator(Operator.OR));
        searchSourceBuilder.query(boolQueryBuilder);

        /**
         * math查询（分词匹配查询），只要与其中一个关键字匹配就可以查出来
         */
       /* MatchQueryBuilder matchQueryBuilder = QueryBuilders.matchQuery(name, value);
        searchSourceBuilder.query(matchQueryBuilder);*/

        // math查询（分词匹配查询）可以支持多个字段检索
        /*QueryBuilder multiMatch = QueryBuilders.multiMatchQuery("zhou", "first_name","last_name");
        searchSourceBuilder.query(multiMatch);*/

        // math查询（分词匹配查询）一定要包含全部关键字
        /*MatchPhraseQueryBuilder matchPhraseQueryBuilder = QueryBuilders.matchPhraseQuery("about", "in this world");
        searchSourceBuilder.query(matchPhraseQueryBuilder);*/

        //构建分页
        if (form == 1) {
            searchSourceBuilder.from();
        } else {
            searchSourceBuilder.from(form);
        }
        searchSourceBuilder.size(size);
        searchSourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
        searchRequest.source(searchSourceBuilder);
        SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
        return search.getHits().getHits();
    }

    /**
     * 精确查找
     *
     * @param index 索引名称
     * @param form  页数 如果查看第一页传1
     * @param size  数量 默认传10
     * @param name  查询参数名
     * @param value 查询值
     * @return 查询好的SearchHit数组
     * @throws IOException es连接异常
     */
    public SearchHit[] searchPage(String index, Integer form, Integer size, String name, String value) throws IOException {
        SearchRequest searchRequest = new SearchRequest(index);
        //构建搜索条件
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //QueryBuilders.termQuery(); //精确查找
        TermQueryBuilder termQueryBuilder = QueryBuilders.termQuery(name, value);
        searchSourceBuilder.query(termQueryBuilder);

        //使用bool检索
        /*bool:几种格式
        must:所有分句都必须匹配，与 AND 相同。
        must_not:所有分句都必须不匹配，与 NOT 相同。
        should: 至少有一个分句匹配，与 OR 相同。
        1 term查询（精准查询）
        2 math查询（分词匹配查询）
        3 fuzzy查询（模糊查询）
        4 wildcard(通配符查询)
        5 bool查询（布尔查询）
        */
        /*BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery()
                .should(QueryBuilders.matchQuery(name,value)
                        .analyzer("ik_max_word")
                        .operator(Operator.OR));
        searchSourceBuilder.query(boolQueryBuilder);*/


        /**
         * math查询（分词匹配查询），只要与其中一个关键字匹配就可以查出来
         */
        /*MatchQueryBuilder matchQueryBuilder = QueryBuilders.matchQuery(name, value);
        searchSourceBuilder.query(matchQueryBuilder);*/

        // math查询（分词匹配查询）可以支持多个字段检索
        /*QueryBuilder multiMatch = QueryBuilders.multiMatchQuery("zhou", "first_name","last_name");
        searchSourceBuilder.query(multiMatch);*/

        // math查询（分词匹配查询）一定要包含全部关键字
        /*MatchPhraseQueryBuilder matchPhraseQueryBuilder = QueryBuilders.matchPhraseQuery("about", "in this world");
        searchSourceBuilder.query(matchPhraseQueryBuilder);*/

        //构建分页
        if (form == 1) {
            searchSourceBuilder.from();
        } else {
            searchSourceBuilder.from(form);
        }
        searchSourceBuilder.size(size);
        searchSourceBuilder.timeout(new TimeValue(60, TimeUnit.SECONDS));
        searchRequest.source(searchSourceBuilder);
        SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
        return search.getHits().getHits();
    }

}
