package indi.felix.easy.core.elastic.model;

import indi.felix.easy.core.elastic.model.metadata.*;
import lombok.Data;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * mapping 实体对象
 * 优化建议：
 * 1. 不需要聚合的字段fielddata 设置为false
 */
@Data
public class MappingModel {
    /**
     * 映射类型包含与文档相关的字段或属性的列表
     */
    private List<BaseFieldMetadata> properties;

    private JoinFieldMetadata joinFieldMetadata;

//    private String _index;
//
//    private String _type;

    /**
     * id 字段
     */
    private String id;

    /**
     * 源ID字段
     */
    private String sourceId;

    /**
     * routing 字段
     */
    private String routingId;

    /**
     * 当 Elasticsearch 遇到文档中以前 未遇到的字段，它用 dynamic mapping 来确定字段的数据类型并自动把新的字段添加到类型映射。
     * <p>
     * true: 动态添加新的字段—​缺省
     * false: 忽略新的字段
     * strict: 如果遇到新字段抛出异常, 把动态类型判断设置为严格, 也就是不允许ES为插入的数据进行动态类型设置, 避免注入脏数据.
     */
    private String dynamic = "false";


    /**
     * text字段类型动态字段后缀模板
     * 由于Elasticsearch支持通过添加数据的形式，动态更新mapping定义，但是有时候，默认推断所创建的mapping是不符合我们的要求的，
     * 这时候可以预先通过定义mapping模板来约束Elasticsearch创建我们想要的mapping。
     * <p>
     * 比如：设置name， dept、后缀的字段自动映射为text
     */
    private List<String> dynamicTextFieldTemp;

    /**
     * 设置自动设置为keyword的字段匹配规则
     */
    private List<String> dynamicKeywordFieldTemp;

    private List<String> dynamicDateFieldTemp;

    private List<String> dynamicTextKeyFieldTemp;


    /**
     * 是否 routing 需要必填，默认否
     */
    private boolean routingRequired;

    public MappingModel() {
        properties = new ArrayList<>();
        joinFieldMetadata = new JoinFieldMetadata();
    }

    public void addMetadata(BaseFieldMetadata metadata) {
        if (metadata == null) {
            return;
        }
        this.properties.add(metadata);
    }

    /**
     * 配置建议
     *
     * @return
     */
    public String suggest(BaseFieldMetadata base) {
        String curType = base.getType();
        String curName = base.getName();
        String suggestType = "";
        if (exist(this.dynamicKeywordFieldTemp, curName)) {
            suggestType = "keyword";
        } else if (exist(this.dynamicTextFieldTemp, curName)) {
            suggestType = "text";
        }

//        switch (curType) {
//            case "text":
//                TextFieldMetadata textFieldMetadata = (TextFieldMetadata) base;
//
//                break;
//            case "keyword":
//                KeywordFieldMetadata keywordFieldMetadata = (KeywordFieldMetadata) base;
//
//                break;
//            case "date":
//            case "datetime":
//            case "time":
//                DateFieldMetadata dateFieldMetadata = (DateFieldMetadata) base;
//
//                break;
//            case "integer":
//            case "int":
//                NumberFieldMetadata integetFieldMetadata = (NumberFieldMetadata) base;
//
//            case "object":
//            case "json":
//                ObjectFieldMetadata jsonField = (ObjectFieldMetadata) base;
//                break;
//            default:
//                break;
//        }

        if (!curType.equals(suggestType)) {
            return "字段[" + curName + "]当前类型为：[" + curType + "], 建议修改为：【" + suggestType + "]";
        }
        return null;
    }

    private static boolean exist(List<String> suffix, String fieldName) {
        fieldName = fieldName.toLowerCase();

        for (String s : suffix) {
            if (fieldName.endsWith(s)) {
                return true;
            }
        }

        return false;
    }

    public void setDefault() {
        // 设置字符串字段创建keyword模板， 自动设置为keyword的字段匹配规则
        if (CollectionUtils.isEmpty(this.dynamicKeywordFieldTemp)) {
            // 设置默认模板
            List<String> dkFields = new ArrayList<>();
            dkFields.add("id");
            dkFields.add("code");
            dkFields.add("no");
            this.setDynamicKeywordFieldTemp(dkFields);
        }

        if (CollectionUtils.isEmpty(this.getDynamicTextFieldTemp())) {
            // 设置默认模板
            List<String> dkFields = new ArrayList<>();
            dkFields.add("text");
            dkFields.add("desc");
            this.setDynamicTextFieldTemp(dkFields);
        }

        if (CollectionUtils.isEmpty(this.getDynamicTextKeyFieldTemp())) {
            // 设置默认模板
            List<String> dkFields = new ArrayList<>();
            dkFields.add("name");
            this.setDynamicTextKeyFieldTemp(dkFields);
        }
    }
}
