package indi.felix.easy.core.elastic.model;

import lombok.Data;

import java.util.HashMap;
import java.util.Map;

/**
 * elasticsearch setting对象
 * ES setting分为 static和dynamic
 * static: 仅可在创建时或关闭时更改
 * dynamic: 可热更新，通过api
 *
 * @author felix
 */
@Data
public class SettingModel {
    /**
     * 分片数, 默认5, static
     */
    private int shards;

    /**
     * 副本数， 默认1, dynamic
     */
    private int replicas;

    /**
     * 内存索引缓冲区的文件被刷到segment中的时间。这段数据会被写入到es的Translog中，可以被检索到，但是尚未刷到磁盘中。
     * Es默认值是1s,这迫使es集群每秒创建一个segment段，增加这个值，可以允许segment更大，并减小以后的segment合并压力。
     */
    private int refreshInterval;

    /**
     * dynamic
     * 分页窗口。默认值10000.超过该值请使用Scroll 或Search After API. 默认使用from + size的情况下的索引
     */
    private int max_result_window;

    private int totalFields;

    public SettingModel() {
        this.shards = 5;
        this.replicas = 1;
        this.totalFields = 5000;
//        this.refreshInterval = 1s
        this.max_result_window = 10000;
    }

    public Map<String, Object> getSettingMap() {
        Map<String, Object> setting = new HashMap<>();

        if (shards >= 0) {
            setting.put("number_of_shards", shards);
        }

        if (replicas >= 0) {
            setting.put("number_of_replicas", replicas);
        }

        if (refreshInterval >= 0) {
            setting.put("index.refresh_interval", refreshInterval);
        }

        if (totalFields >= 0) {
            setting.put("index.mapping.total_fields.limit", totalFields);
        }

        if (max_result_window >= 0) {
            setting.put("index.max_result_window", max_result_window);
        }

        return setting;
    }
}
