package indi.felix.easy.core.elastic.rest;

import indi.felix.easy.core.elastic.client.RequestHelper;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.nio.entity.NStringEntity;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

/**
 * REST API 参看：https://www.kancloud.cn/apachecn/elasticsearch-doc-zh/1945128
 *
 * @author houpengpeng
 */
public abstract class BaseRest {
    protected static final Logger log = LoggerFactory.getLogger(BaseRest.class);
    protected RestClient restClient;

    public static final String
            GET = "GET", POST = "POST", PUT = "PUT", DELETE = "DELETE", HEAD = "HEAD";

    protected BaseRest(RestClient restClient) {
        this.restClient = restClient;
    }

    public String responseBody(String method, String endpoint) {
        return response(method, endpoint, Collections.EMPTY_MAP, null);
    }

    public String responseBody(String method, String endpoint, Map<String, String> params) {
        return response(method, endpoint, params, null);
    }

    public String responseBody(String method, String endpoint, Map<String, String> params, Header[] headers) {
        return response(method, endpoint, params, headers);
    }

    public String responseBody(String method, String endpoint, String dsl) {
        return response(method, endpoint, null, null, dsl);
    }

    public String responseBody(String method, String endpoint, Map<String, String> params, org.apache.http.HttpEntity entity) {
        Response response = null;
        String responseBody = null;
        try {
            Request request = RequestHelper.getRequest(method, endpoint, params, entity);
            response = restClient.performRequest(request);
            responseBody = EntityUtils.toString(response.getEntity(), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return responseBody;
    }

    private String response(String method, String endpoint, Map<String, String> params, Header[] headers, String dsl) {
        Response response = null;
        String responseBody = null;
        try {
            Request request = RequestHelper.getRequest(method, endpoint, params, headers);

            if (StringUtils.isNotEmpty(dsl)) {
                request.setEntity(new NStringEntity(
                        dsl,
                        ContentType.APPLICATION_JSON));
            }

            response = restClient.performRequest(request);
            responseBody = EntityUtils.toString(response.getEntity(), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return responseBody;
    }

    private String response(String method, String endpoint, Map<String, String> params, Header[] headers) {
        Response response = null;
        String responseBody = null;
        try {
            Request request = RequestHelper.getRequest(method, endpoint, params, headers);
            response = restClient.performRequest(request);
            responseBody = EntityUtils.toString(response.getEntity(), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return responseBody;
    }

    protected Response res(String method, String endpoint, Map<String, String> params, Header[] headers) {
        Response response = null;
        try {
            Request request = RequestHelper.getRequest(method, endpoint, params, headers);
            response = restClient.performRequest(request);

        } catch (IOException e) {
            e.printStackTrace();
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return response;
    }

    protected Response resBody(String method, String endpoint, Map<String, String> params, HttpEntity entity) {
        Response response = null;
        try {
            Request request = RequestHelper.getRequest(method, endpoint, params, entity);
            response = restClient.performRequest(request);

        } catch (IOException e) {
            e.printStackTrace();
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return response;
    }

}
