package indi.felix.easy.core.elastic.rest;

import indi.felix.easy.core.elastic.client.XContentUtils;
import indi.felix.easy.core.elastic.model.ESFieldTypeEnum;
import indi.felix.easy.core.elastic.model.ElasticConstant;
import org.apache.http.HttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.nio.entity.NStringEntity;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.common.bytes.BytesReference;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.common.xcontent.XContentHelper;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.Collections;

import static indi.felix.easy.core.elastic.utool.Const.DOC_TYPE;
import static org.elasticsearch.common.xcontent.ToXContent.EMPTY_PARAMS;
import static org.elasticsearch.common.xcontent.XContentFactory.jsonBuilder;
import static org.elasticsearch.common.xcontent.XContentHelper.toXContent;

public class IndexRestApi extends BaseRest {
    public IndexRestApi(RestClient restClient) {
        super(restClient);
    }

    String defaultMethod = GET;
    String[] methods = new String[]{GET, PUT, HEAD};
    String endpoint = "_indices";

    /**
     * 查看集群状态 api
     * curl -XHEAD 'localhost:9200/twitter
     *
     * @return
     */
    public boolean exists(String index) {
        Response response = res(HEAD, index, Collections.EMPTY_MAP, null);
        int code = response.getStatusLine().getStatusCode();
        return code == 200;
    }

    /**
     * Get Index API允许获得一个或多个索引的信息
     * curl -XGET 'localhost:9200/twitter
     *
     * @param index
     * @return
     */
    public String getIndex(String index) {
        return responseBody(defaultMethod, index);
    }

    /**
     * curl -XGET 'localhost:9200/_mapping
     *
     * @return
     */
    public String getMapping() {
        String newEndpoint = "_mapping";
        return responseBody(defaultMethod, newEndpoint);
    }


    /**
     * curl -XGET 'localhost:9200/twitter/_mapping
     *
     * @return
     */
    public String getMapping(String index) {
        String newEndpoint = index + "/_mapping";
        return responseBody(defaultMethod, newEndpoint);
    }

    /**
     * curl -XPUT 'localhost:9200/twitter/_mapping
     *
     * @return
     */
    public String putMapping(String index, String body) {
        String newEndpoint = MessageFormat.format("/{0}/_mapping/{1}", index, DOC_TYPE);

        return responseBody(PUT, newEndpoint, body);
    }

    /**
     * 添加属性
     *
     * @param index
     * @param propertiesName 属性字段
     * @param propertiesType 属性类型
     * @return
     */
    public String putMapping(String index, String propertiesName, ESFieldTypeEnum propertiesType) {
        XContentBuilder builder = null;
        try {
            builder = XContentFactory.jsonBuilder().prettyPrint();
            builder.startObject();
            builder.startObject("properties");
            XContentUtils.setMappingField(builder, propertiesName, propertiesType);
            builder.endObject();
            builder.endObject();
            String str = XContentHelper.convertToJson(BytesReference.bytes(builder), true, builder.contentType());

            return putMapping(index, str);
        } catch (IOException e) {
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return null;
    }

    public String putDateProperty(String index, String field) {
        String newEndpoint = index + "/_mapping/" + DOC_TYPE + "?pretty";
        String jsonStr = "{\n" +
                "  \"properties\": {\n" +
                "    \" " + field + "\": {\n" +
                "      \"type\": \"date\"\n" +
                "    }\n" +
                "  }\n" +
                "}";
        HttpEntity entity = new NStringEntity(jsonStr, ContentType.APPLICATION_JSON);

        return responseBody(PUT, newEndpoint, Collections.emptyMap(), entity);
    }

}
