package indi.felix.easy.core.elastic.rest;

import indi.felix.easy.core.elastic.client.XContentUtils;
import org.apache.http.HttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.nio.entity.NStringEntity;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.common.bytes.BytesReference;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;
import org.elasticsearch.common.xcontent.XContentHelper;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;

import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Collections;

import static indi.felix.easy.core.elastic.utool.Const.DOC_TYPE;

public class SearchRestApi extends BaseRest {
    private SearchRest rest;
    public SearchRestApi(RestClient restClient) {
        super(restClient);
        rest = new SearchRest();
    }

    String defaultMethod = GET;
    String[] methods = new String[]{GET, POST};
    String endpoint = "_search";

    public SearchRest getRest(){
        return rest;
    }

    /**
     * GET /_search 查询
     *
     * @param index 多个索引查询用逗号分割
     * @param dsl   DSL 语句字符串
     * @return
     */
    public String requestDsl(String index, String dsl) {
        String newEndpoint = MessageFormat.format("/{0}/_search", index);

        return responseBody(defaultMethod, newEndpoint, dsl);
    }

    /**
     * 查询指定ID的文档
     *
     * @param index 多个索引查询用逗号分割
     * @param id
     * @return
     */
    public String requestById(String index, String id) {
        String newEndpoint = MessageFormat.format("/{0}/{1}", index, id);

        return responseBody(defaultMethod, newEndpoint);
    }

    /**
     * 查询多个ids的文档
     * GET index/type/_mget
     * {
     *   "ids":[12,13]
     * }
     *
     * @param index 多个索引查询用逗号分割
     * @param ids
     * @return
     */
    public String requestByIds(String index, String... ids) {
        String newEndpoint = MessageFormat.format("/{0}/{1}", index, "_mget");

        XContentBuilder builder = null;
        try {
            builder = XContentFactory.jsonBuilder().prettyPrint();
            builder.startObject();
            builder.field("ids", ids);
            builder.endObject();
            String str = XContentHelper.convertToJson(BytesReference.bytes(builder), true, builder.contentType());
            HttpEntity entity = new NStringEntity(str, ContentType.APPLICATION_JSON);

            return responseBody(GET, newEndpoint, Collections.emptyMap(), entity);
        }catch (Exception e){
            log.error(">>> easy-es 请求出错，错误信息：{}", e.getMessage());
        }

        return null;
    }

    /**
     * 根据查询条件查询指定索引文档
     *
     * @param index        多个索引查询用逗号分割
     * @param queryBuilder ES 查询条件
     * @return
     */
    public String requestByQuery(String index, SearchSourceBuilder queryBuilder) {
        return requestDsl(index, queryBuilder.toString());
    }

    /**
     * 根据查询条件查询指定索引文档
     * @param index
     * @param queryBuilder
     * @return
     */
    public String requestByQuery(String index, QueryBuilder queryBuilder) {
        SearchSourceBuilder builder = new SearchSourceBuilder();
        builder.query(queryBuilder);

        String jsonStr = builder.toString();

        return requestDsl(index, jsonStr);
    }

    public class SearchRest {
        private String[] defaultIndex;
        private SearchRestApi restApi;

        public void setDefaultIndex(String... index) {
            this.defaultIndex = index;
        }

        public String[] getDefaultIndex() {
            return this.defaultIndex;
        }

        public String requestById(String id) {
            String indexs = String.join(",", defaultIndex);
            String newEndpoint = MessageFormat.format("/{0}/{2}", indexs, id);

            return responseBody(defaultMethod, newEndpoint);
        }
    }
}

