package indi.felix.easy.core.elastic.rest;

import indi.felix.easy.core.elastic.EasyEs;
import indi.felix.easy.core.elastic.model.ESFieldTypeEnum;
import junit.framework.TestCase;
import org.junit.Assert;

import java.util.HashMap;
import java.util.Map;

public class EasyRestTest extends TestCase {
    private static final String ip = "localhost:9200";

    public void testPing() {
        EasyEs easyEs = new EasyEs(ip);

        // 测试ES是否能够连接通
        Assert.assertTrue(easyEs.rest().ping());

        String testIndexName = "test_20210907";
        if (easyEs.rest().index().exists(testIndexName)) {
            Assert.assertTrue(easyEs.rest().deleteIndex(testIndexName));
            Assert.assertFalse(easyEs.rest().index().exists(testIndexName));
        }
        easyEs.rest().createIndex(testIndexName);
        Assert.assertTrue(easyEs.rest().index().exists(testIndexName));

        Map<String, String> map = new HashMap<>();
        map.put("name", "张三");
        map.put("age", "12");
        // 插入文档
        easyEs.rest().document().addDoc(testIndexName, map);

        Map<String, String> map1 = new HashMap<>();
        map1.put("name", "李四");
        map1.put("age", "11");

        easyEs.rest().document().addDoc(testIndexName, "123456", map1);

        Map<String, String> map2 = new HashMap<>();
        map2.put("name", "王五");
        map2.put("age", "10");

        easyEs.rest().document().addDoc(testIndexName, "123457", "123457", map2);

        Assert.assertEquals("{\"_index\":\"test_20210907\",\"_type\":\"doc\",\"_id\":\"123456\",\"_version\":1,\"found\":true,\"_source\":{\"name\":\"李四\",\"age\":\"11\"}}",
                easyEs.rest().search().requestById(testIndexName, "123456"));


        easyEs.rest().document().deleteDoc(testIndexName, "123457");

        Assert.assertTrue(easyEs.rest().deleteIndex(testIndexName));
        Assert.assertFalse(easyEs.rest().index().exists(testIndexName));
    }

    public void testIndex() {
        EasyEs easyEs = new EasyEs(ip);
        String testIndexName = "test_20210907";

        easyEs.rest().createIndex(testIndexName);
        // 获取全部索引
        String allMapping = easyEs.rest().index().getMapping();
        System.out.println(allMapping);
        String mapping = easyEs.rest().index().getMapping(testIndexName);
        Assert.assertEquals("{\"test_20210907\":{\"mappings\":{\"doc\":{}}}}",  mapping);


        //putMapping test
        String putMapping = easyEs.rest().index().putMapping(testIndexName, "{\n" +
                "  \"properties\": {\n" +
                "    \"name\": {\n" +
                "      \"type\": \"text\"\n" +
                "    }\n" +
                "  }\n" +
                "}");

        String pMapping = easyEs.rest().index().getMapping(testIndexName);
        System.out.println(pMapping);

        String pMapping11 = easyEs.rest().index()
                .putMapping(testIndexName, "name11", ESFieldTypeEnum.KEYWORD);
        System.out.println(easyEs.rest().index().getMapping(testIndexName));

        easyEs.rest().deleteIndex(testIndexName);
    }
}
